<?php
##
## Copyright (c) 1999 Internet Images srl
##                    Massimiliano Masserelli
##
## $Id: ct_split_sql.inc,v 1.1.2.1 2000/04/09 19:50:32 chuck Exp $
##
## PHPLIB Data Storage Container using a SQL database and multiple
## rows for each element
##
## Every session-name pair will end up in one OR MORE table rows, thus
## allowing serialization of huge quantities of data.
##

class CT_Split_Sql {
    ##
    ## Define these parameters by overwriting or by
    ## deriving your own class from it (recommened)
    ##
    
    var $database_table = "active_sessions_split";
    var $database_class = "";
    var $database_lock_semaphore = "";
    var $split_length = 4096;  ## Split data every xxx bytes

    var $enc_methods = array(
      "base64"  => array("enc" => "base64_encode", "dec" => "base64_decode"),
      "slashes" => array("enc" => "addslashes",    "dec" => "strval")
    );
    var $encoding_mode = "base64";
    ## end of configuration

    var $db;

    function ac_start() {
        $name = $this->database_class;
        $this->db = new $name;
    }

    function ac_get_lock() {
        if ( "" != $this->database_lock_semaphore ) {
            while ( ! $this->db->query("SELECT get_lock('%s')", 
                        $this->database_lock_semaphore) ) {
                $t = 1 + time(); while ( $t > time() ) { ; }
            }
        }
    }

    function ac_release_lock() {
        if ( "" != $this->database_lock_semaphore ) {
            $this->db->query("SELECT release_lock('%s')", 
                    $this->database_lock_semaphore);
        }
    }

    function ac_gc($gc_time, $name) {
        $timeout = time();
        $sqldate = date("YmdHis", $timeout - ($gc_time * 60));
        $this->db->query(sprintf("DELETE FROM %s ".
                    "WHERE ct_changed < '%s' AND ct_name = '%s'",
                    $this->database_table, 
                    $sqldate,
                    addslashes($name)));
    }

    function ac_store($id, $name, $str) {
        $ret = true;
        
        $cmd  = $this->enc_methods[$this->encoding_mode]["enc"];
        $str  = sprintf("%s:%s", $this->encoding_mode, $cmd($str));
        
        $now = date("YmdHis", time());
        
        if ( $this->db->type == "oracle" )
        {
            Ora_CommitOff($this->db->Link_ID);
        } else {
            $this->db->query("BEGIN TRANSACTION");
        }
        $this->ac_delete($id, $name);
        
        $count = 0;
        while ($part = substr($str, 0, $this->split_length)) {
            $this->db->query(sprintf("INSERT INTO %s ".
                " (ct_sid, ct_name, ct_pos, ct_val, ct_changed) ".
                "  VALUES ".
                " ('%s','%s','%06d','%s','%s')",
                $this->database_table,
                $id,
                addslashes($name),
                $count++,
                $part,
                $now
            ));
            $str = substr($str, $this->split_length);
        }
        if ( $this->db->type == "oracle" )
        {
            // Commit transaction
            if(!@Ora_Commit($this->db->Link_ID))
            {
                $ret = false;
            }
            // Enable autocommit again            
            Ora_CommitOn($this->db->Link_ID);
        } else {
            $this->db->query("END TRANSACTION");
        }
        return $ret;
    }

    function ac_delete($id, $name) {
        $this->db->query(sprintf("DELETE FROM %s ".
                    "WHERE ct_name = '%s' AND ct_sid = '%s'",
                    $this->database_table,
                    addslashes($name),
                    $id));
    }

    function ac_get_value($id, $name) {
        $this->db->query(sprintf("SELECT ct_val, ct_pos FROM %s ".
                        "WHERE ct_sid  = '%s' AND ct_name = '%s' ".
                        "ORDER BY ct_pos",
                    $this->database_table,
                    $id,
                    addslashes($name)));
        $str="";
        while ($this->db->next_record()) {
          $str .= $this->db->f("ct_val");
        }

        # get encoding method from (method:value) pair
        $colon_pos = strpos($str,':');
        $str_method = substr ( $str , 0 , $colon_pos );
        if( empty($str_method) ) { 
            // For compatibility with old sessions
            $str_method = "base64";
            $str_value  = $str;
        } else {
            // New format case
            $str_value  = substr ( $str , $colon_pos + 1 );
        }
        $cmd  = $this->enc_methods[$str_method]["dec"];

        $str  = $cmd($str_value);
        
## DEB        echo $str;
        return $str;
    }

    function ac_newid($str, $name) {
        return $str;
    }

    function ac_halt($s) {
        $this->db->halt($s);
    }
   
    ## provide your own encoding/decoding method(s)
    function set_enc_meth($name, $enc_func, $dec_func) {
      if (!isset($this->enc_methods[$name])) {
        $this->enc_methods[$name] = array ('enc'=> $enc_func, 'dec'=> $dec_func) ;
      }
    }

    function get_enc_meth($name) {
      return $this->enc_methods[$name];
    }
}
?>
