<?php
/*
  File: config/mime.conf
  $Author: jon $
  $Revision: 2.0.2.1 $
  $Date: 1999/12/28 02:21:17 $
  
  IMP: Copyright 1999 The Horde Project
  
  This code is under the GNU Public License
  See the file COPYING in this directory
*/

/*
 * MIME Type Configuration
 *
 * This file allows you to configure site-specific MIME types and control
 * how HTML email is rendered by IMP.  Please read the comments carefully if
 * you change it.
 *
 * If you'd like IMP to handle HTML email, please examine the last three
 * examples.
 *
 * If you're adding your own type, each entry is an array, and here are what
 * the parts control:
 *
 *  $mime_actions["type/subtype"]   The MIME type of the following section
 *
 *      "action"                    Determines whether a special driver needs
 *                                  to be called for this type.  These drivers
 *                                  are implemented in the mimetypes.lib file.
 *                                  Usually "default".
 *
 *      "view"                      Determines whether the MIME type is viewable
 *                                  in a stand-alone window.  This is useful for
 *                                  large things like Word documents displayed
 *                                  via mswordview.
 *
 *      "inline"                    Determines whether IMP should display this
 *                                  type in the main message window.  This is
 *                                  useful for images, HTML, or other small
 *                                  things.
 *
 *      "override_text"             Determines if this MIME type will display in
 *                                  place of other MIME types.  This option is
 *                                  not necessarily safe, but can be used well
 *                                  for things like displaying HTML parts of
 *                                  email without the non-Rich part. (optional)
 *
 *      "download"                  Determines whether this MIME type is
 *                                  separately downloadable from IMP.  This is
 *                                  useful for almost everything, such as
 *                                  documents, images, tarballs, etc.
 *
 *      "icon"                      The icon to use for this MIME type.  Icons
 *                                  are located in <imp>/graphics/. (optional)
 *
 *      "view_function"             The name of any special MIME driver needed
 *                                  to view this MIME type. (optional)
 *
 *      "function"                  The name of any special MIME driver needed
 *                                  to download this MIME type.  This is only
 *                                  required if "action" is set to "function".
 */

$mime_actions["application/pgp"] = array(
    "action"        => "default",
    "view"          => true,
    "inline"        => true,
    "view_function" => "mime_view_text",
    "download"      => true,
    "icon"          => "mime_text.gif"
);


$mime_actions["image/"] = array(
    "action"        => "function",
    "function"      => "mime_action_images",
    "view"          => true,
    "download"      => true,
    "icon"          => "mime_image.gif"
);

$mime_actions["application/msword"] = array(
    "action"        => "default",
    "view"          => true,
    "view_function" => "mime_view_msword",
    "download"      => true,
    "icon"          => "mime_msword.gif"
);

$mime_actions["application/x-imp-data"] = array(
    "action"        => "function",
    "function"      => "mime_action_ximpdata",
    "view"          => false,
    "download"      => false,
    "icon"          => "mime_broken.gif"
);

$mime_actions['application/x-gtar'] = array(
    "action"        => "default",
    "view"          => true,
    "view_function" => "mime_view_tgz",
    "download"      => true,
    "icon"          => "mime_compressed.gif"
);


/*
 * HTML email viewing
 *
 * There are multiple ways to configure IMP to view HTML sent as email.
 * Here are three.
 *
 * Note:  It is VERY IMPORTANT to note that if you configure HTML to be
 * viewed inline, it will OVERRIDE the text part of the email and replace it
 * with the HTML part.  This follows the behavior of most "rich" email
 * clients that send HTML mail with a text and HTML part that are identical,
 * except for the mark-up.
 *
 * If you don't uncomment any of these sections, HTML parts are viewed just
 * like any other attachment.
 */

/*
 * Option 1:
 *
 *  Allow the HTML MIME-part to be viewed in another browser window, allow
 *  it to be downloaded, and make it inline.  This is the safest way to read
 *  mail and still have HTML mail render-able.  All options are open.
 */
#$mime_actions["text/html"] = array(
#    "action"        => "default",
#    "view"          => true,
#    "inline"        => true,
#    "override_text" => false,
#    "view_function" => "mime_view_html",
#    "download"      => true,
#    "icon"          => "mime_html.gif"
#);

/*
 * Option 2:
 *
 *  View the HTML MIME-part inline, overriding the text part, and let it be
 *  downloaded.  This is not provably 100% safe (you may lose text), but we
 *  do sanitize the HTML for security.
 */
#$mime_actions["text/html"] = array(
#    "action"        => "default",
#    "view"          => true,
#    "inline"        => true,
#    "override_text" => true,
#    "view_function" => "mime_view_html",
#    "download"      => true,
#    "icon"          => "mime_html.gif"
#);

/*
 * Option 3:
 *
 *  This is my (mike@itsprojects.com) favorite - HTML inline, not
 *  downloadable.  Overrides text parts.  Totally transparent.  This is just
 *  like the previous one except the user interface is less cluttered
 *  without the download option.  There is the potential danger that we lose
 *  some text parts when we do this, but current HTML email clients send the
 *  email in such a way that this doesn't happen as far as I know.
 */
#$mime_actions["text/html"] = array(
#    "action"        => "default",
#    "view"          => true,
#    "inline"        => true,
#    "override_text" => true,
#    "view_function" => "mime_view_html",
#    "download"      => false,
#    "icon"          => "mime_html.gif"
#);

?>
